AZURE <- TRUE
NUM_SAMPLES <- 1000

library(dplyr)
library(tidyr)
library(readr)
library(purrr)

# data --------------------------------------------------------------------

if (AZURE) {
  calls <- maml.mapInputPort(1)
} else {
  calls <- read_csv('processed/calls.csv', col_types = cols())
}

calls <- calls %>% 
  filter(!is.na(hotline_start_time), !is.na(hotline_end_time))
  
# functions ---------------------------------------------------------------

extract_time <- function(x) {
  format <- "%H:%M:%S"
  #print(x)
  #print(strftime(x, format=format))
  as.POSIXct(strftime(x, format=format), format=format)
}

as_yearly_period <- function(x) {
  x <- (as.numeric(x) - as.numeric(as.Date("2010-01-01"))) / 365 * 2 * pi
}


estimate_max_used <- function(num_calls) {
  print(map(calls, typeof))
  calls %>% 
    select(start = hotline_start_time, end = hotline_end_time) %>% 
    sample_n(num_calls) %>% 
    gather('type', 'time') %>% 
    mutate(time = extract_time(time)) %>% 
    arrange(time) %>% 
    mutate(value = ifelse(type == 'start', 1L, -1L),
           num_used = cumsum(value)) %>% 
    summarise(num_used = max(num_used)) %>% 
    pull(num_used)
}


# simulation --------------------------------------------------------------

required_operators <- crossing(num_calls = c(seq(1, 29, 2), seq(30, 100, 5), seq(110, 250, 10)), 
                      repl = seq_len(NUM_SAMPLES)) %>% 
  mutate(max_used = map_dbl(num_calls, estimate_max_used)) %>% 
  group_by(num_calls) %>% 
  summarise(q = quantile(max_used, probs = 0.85)) %>%
  #summarise(q = list(quantile(max_used, probs = c(0.8, 0.9, 0.95, 0.99)))) %>% 
  #unnest_wider(q) %>% 
  #set_names(gsub, pattern = "([0-9]+)\\%", replacement = "q_\\1") %>% 
  complete(num_calls = seq(min(num_calls), max(num_calls))) %>% 
  mutate_at(vars(-num_calls), round) %>% 
  mutate_at(vars(-num_calls), zoo::na.locf) %>% 
  mutate_at(vars(-num_calls), cummax)
  
# ggplot(df, aes(num_calls, q_90)) +
#   geom_point()

if (AZURE) {
  maml.mapOutputPort("required_operators")
} else {
  write_csv(simulated, 'processed/required_operators.csv')
}


